import datetime
import logging
from typing import List

import typer
from alvin_api_client.model.data_entity_id import DataEntityID
from alvin_api_client.model.data_entity_lineage_v2_request import (
    DataEntityLineageV2Request,
)
from alvin_api_client.model.data_entity_type import DataEntityType

from alvin_cli.schemas.models import OutputFormat
from alvin_cli.utils import default_api
from alvin_cli.utils.helper_functions import extract_dict
from alvin_cli.utils.helper_functions import format_response_data
from alvin_cli.utils.helper_functions import handle_print_exception
from alvin_cli.utils.helper_functions import print_output_format
from alvin_cli.utils.helper_functions import structure_data_usage_stats
from alvin_cli.utils.helper_functions import structure_lineage_node_data

app = typer.Typer()


def __setup_logging() -> None:
    logging.basicConfig(level=logging.INFO)


start_timestamp = datetime.datetime.now() - datetime.timedelta(30)
end_timestamp = datetime.datetime.now()

fields_to_print_entity_children_and_parent = [
    "name",
    "id",
    "entity_type",
    "data_type",
    "is_leaf_entity",
    "is_top_level_entity",
    "parent_id",
    "parent_type",
    "created_time",
    "updated_time",
    "platform_td",
    "platform_type",
    "platform_created_time",
    "platform_updated_time",
    "facet",
]


@app.command()
def get(
    platform_id: str = typer.Option(
        ...,
        help=typer.style(
            "Platform ID To Get Details", fg=typer.colors.BRIGHT_GREEN, bold=True
        ),
    ),
    entity_id: str = typer.Option(
        ..., help=typer.style("Entity ID", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    entity_type: str = typer.Option(
        ..., help=typer.style("Entity Type", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    output: OutputFormat = typer.Option(
        OutputFormat.table,
        help=typer.style(
            "Format to receive output in, default is table", fg=typer.colors.YELLOW
        ),
    ),
    save_to_file: bool = typer.Option(
        False, help=typer.style("Save data to a file?", fg=typer.colors.YELLOW)
    ),
    file_name: str = typer.Option(
        "entity_get",
        help=typer.style(
            "File Name if file save selected. 'entity_get.* by default",
            fg=typer.colors.YELLOW,
        ),
    ),
) -> None:
    """Get Entity Info"""

    title = "Entity Data"

    try:
        entity_response = default_api.get_entity_api_v1_entity_get(
            platform_id=platform_id,
            entity_id=entity_id,
            entity_type=DataEntityType(entity_type.upper()),
        )
        fields_to_print = [
            "id",
            "name",
            "parent_id",
            "created_time",
            "entity_type",
            "platform_id",
            "platform_type",
            "platform_created_time",
            "platform_updated_time",
            "updated_time",
            "entity_link_to_alvin",
        ]
        if entity_response:
            structured_data = format_response_data(fields_to_print, [entity_response])
            print_output_format(structured_data, title, output, save_to_file, file_name)
        else:
            typer.secho("Check all the arguments are valid", fg=typer.colors.RED)

    except Exception as e:
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return


@app.command()
def children(
    platform_id: str = typer.Option(
        ...,
        help=typer.style(
            "Platform ID To Get Details", fg=typer.colors.BRIGHT_GREEN, bold=True
        ),
    ),
    entity_id: str = typer.Option(
        ..., help=typer.style("Entity ID", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    entity_type: str = typer.Option(
        ..., help=typer.style("Entity Type", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    has_connections: bool = typer.Option(
        False, help=typer.style("Does it have connections?", fg=typer.colors.YELLOW)
    ),
    limit: int = typer.Option(
        10, help=typer.style("Limit, 0 by default", fg=typer.colors.YELLOW)
    ),
    offset: int = typer.Option(
        0, help=typer.style("Max Levels, 10 by default", fg=typer.colors.YELLOW)
    ),
    output: OutputFormat = typer.Option(
        OutputFormat.table,
        help=typer.style(
            "Format to receive output in, default is table", fg=typer.colors.YELLOW
        ),
    ),
    save_to_file: bool = typer.Option(
        False, help=typer.style("Save data to a file?", fg=typer.colors.YELLOW)
    ),
    file_name: str = typer.Option(
        "entity_children",
        help=typer.style(
            "File Name if file save selected. 'entity_children.* by default",
            fg=typer.colors.YELLOW,
        ),
    ),
) -> None:
    """Get Entity Children"""

    title = "Entity Children"
    try:
        entities_children = default_api.get_entity_children_api_v1_entity_children_get(
            platform_id=platform_id,
            entity_id=entity_id,
            entity_type=entity_type.upper(),
            has_connections=has_connections,
            limit=limit,
            offset=offset,
        )

        if entities_children and entities_children["items"]:
            structured_data = format_response_data(
                fields_to_print_entity_children_and_parent, entities_children["items"]
            )
            print_output_format(structured_data, title, output, save_to_file, file_name)
        else:
            typer.secho("Data Not Found, check valid arguments", fg=typer.colors.RED)

    except Exception as e:
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return


@app.command()
def parent(
    platform_id: str = typer.Option(
        ...,
        help=typer.style(
            "Platform ID To Get Details", fg=typer.colors.BRIGHT_GREEN, bold=True
        ),
    ),
    entity_id: str = typer.Option(
        ..., help=typer.style("Entity ID", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    entity_type: str = typer.Option(
        ..., help=typer.style("Entity Type", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    output: OutputFormat = typer.Option(
        OutputFormat.table,
        help=typer.style(
            "Format to receive output in, default is table", fg=typer.colors.YELLOW
        ),
    ),
    save_to_file: bool = typer.Option(
        False, help=typer.style("Save data to a file?", fg=typer.colors.YELLOW)
    ),
    file_name: str = typer.Option(
        "entity_parent",
        help=typer.style(
            "File Name if file save selected. 'entity_parent.* by default",
            fg=typer.colors.YELLOW,
        ),
    ),
) -> None:
    """Get Entity Parent"""

    title = "Entity Parent"

    try:
        entities_parent = default_api.get_entity_parents_api_v1_entity_parents_get(
            platform_id=platform_id,
            entity_id=entity_id,
            entity_type=entity_type.upper(),
        )

        if entities_parent:
            structured_data = format_response_data(
                fields_to_print_entity_children_and_parent, entities_parent
            )
            print_output_format(structured_data, title, output, save_to_file, file_name)
        else:
            typer.secho("Data Not Found, check valid arguments", fg=typer.colors.RED)

    except Exception as e:
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return


@app.command()
def lineage(
    platform_id: str = typer.Option(
        ..., help=typer.style("Platform ID", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    entity_id: str = typer.Option(
        ..., help=typer.style("Entity Id", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    entity_type: str = typer.Option(
        ..., help=typer.style("Entity Type", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    lineage_direction: str = typer.Option(
        "downstream",
        help=typer.style("Upstream or Downstream?", fg=typer.colors.YELLOW),
    ),
    limit: int = typer.Option(
        0, help=typer.style("Limit, 0 by default", fg=typer.colors.YELLOW)
    ),
    max_levels: int = typer.Option(
        10, help=typer.style("Max Levels, 10 by default", fg=typer.colors.YELLOW)
    ),
    output: OutputFormat = typer.Option(
        OutputFormat.table,
        help=typer.style(
            "Format to receive output in, default is table", fg=typer.colors.YELLOW
        ),
    ),
    save_to_file: bool = typer.Option(
        False, help=typer.style("Save data to a file?", fg=typer.colors.YELLOW)
    ),
    file_name: str = typer.Option(
        "entity_lineage",
        help=typer.style(
            "File Name if file save selected. 'entity_lineage.* by default",
            fg=typer.colors.YELLOW,
        ),
    ),
) -> None:
    """Get Entity Lineage Node Data"""

    title = "Entity Lineage Node Data"
    upstream_bool = True if lineage_direction == "upstream" else False

    request_in = DataEntityLineageV2Request(
        source_entities=[
            DataEntityID(
                platform_id=platform_id,
                entity_type=DataEntityType(entity_type.upper()),
                entity_id=entity_id,
            )
        ],
        upstream=upstream_bool,
        limit=limit,
        max_levels=max_levels,
    )

    try:
        lineage_data = default_api.get_entity_lineage_api_v2_lineage_post(request_in)
        if lineage_data and lineage_data["nodes"]:
            (structured_node_data) = structure_lineage_node_data(lineage_data["nodes"])
            print_output_format(
                structured_node_data, title, output, save_to_file, file_name
            )
        else:
            typer.secho("Check all the arguments are valid", fg=typer.colors.RED)

    except Exception as e:
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return


@app.command()
def usage(
    platform_id: str = typer.Option(
        ..., help=typer.style("Platform Id", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    entity_type: str = typer.Option(
        ..., help=typer.style("Entity Type", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    entity_id: str = typer.Option(
        ..., help=typer.style("Entity Id", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    start: str = typer.Option(
        start_timestamp,
        help=typer.style(
            "Stats shown for 30 days by default, edit start for other time window",
            fg=typer.colors.YELLOW,
        ),
    ),
    end: str = typer.Option(
        end_timestamp,
        help=typer.style(
            "Stats shown for 30 days by default, edit stop for other time window",
            fg=typer.colors.YELLOW,
        ),
    ),
    usage_type: List[str] = typer.Option(
        [""], help=typer.style("Usage Types", fg=typer.colors.YELLOW)
    ),
    user_name: List[str] = typer.Option(
        [""], help=typer.style("User Name", fg=typer.colors.YELLOW)
    ),
    output: OutputFormat = typer.Option(
        OutputFormat.table,
        help=typer.style(
            "Format to receive output in, default is table", fg=typer.colors.YELLOW
        ),
    ),
    save_to_file: bool = typer.Option(
        False, help=typer.style("Save data to a file?", fg=typer.colors.YELLOW)
    ),
    file_name: str = typer.Option(
        "entity_usage",
        help=typer.style(
            "File Name if file save selected. 'entity_usage.* by default",
            fg=typer.colors.YELLOW,
        ),
    ),
) -> None:
    """Entity Usage Stats for 30 days by default"""

    title = "Entity Usage Stats"
    try:
        entity_usage = default_api.get_entity_usage_stats_api_v1_entity_usage_stats_get(
            platform_id=platform_id,
            entity_id=entity_id,
            entity_type=[entity_type.upper()],
            start_timestamp=start,
            end_timestamp=end,
            usage_type=usage_type,
            user_name=user_name,
        )

        if entity_usage:
            typer.secho(
                "Usage Stats Data with following details",
                fg=typer.colors.BRIGHT_MAGENTA,
            )
            (
                usage_stats_in_list,
                user_name_stats,
            ) = structure_data_usage_stats(entity_usage)
            print_output_format(
                usage_stats_in_list, title, output, save_to_file, file_name
            )
            print_output_format(
                user_name_stats, "User Stats", output, save_to_file, file_name
            )

        else:
            typer.secho("Check all the arguments are valid", fg=typer.colors.RED)

    except Exception as e:
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return
