import json

import typer
from alvin_api_client.model.big_query_account_details import BigQueryAccountDetails
from alvin_api_client.model.big_query_additional_config import BigQueryAdditionalConfig
from alvin_api_client.model.data_platform_big_query_create import (
    DataPlatformBigQueryCreate,
)
from alvin_api_client.model.data_platform_looker_create import DataPlatformLookerCreate
from alvin_api_client.model.data_platform_mode_create import DataPlatformModeCreate
from alvin_api_client.model.data_platform_redshift_create import (
    DataPlatformRedshiftCreate,
)
from alvin_api_client.model.data_platform_snowflake_create import (
    DataPlatformSnowflakeCreate,
)
from alvin_api_client.model.data_platform_tableau_create import (
    DataPlatformTableauCreate,
)
from alvin_api_client.model.looker_additional_config import LookerAdditionalConfig
from alvin_api_client.model.looker_credentials import LookerCredentials
from alvin_api_client.model.mode_additional_config import ModeAdditionalConfig
from alvin_api_client.model.mode_credentials import ModeCredentials
from alvin_api_client.model.redshift_account_details import RedshiftAccountDetails
from alvin_api_client.model.redshift_additional_config import RedshiftAdditionalConfig
from alvin_api_client.model.snowflake_account_details import SnowflakeAccountDetails
from alvin_api_client.model.snowflake_additional_config import SnowflakeAdditionalConfig
from alvin_api_client.model.tableau_additional_config import TableauAdditionalConfig
from alvin_api_client.model.tableau_credentials import TableauCredentials

from alvin_cli.utils.api_client import default_api
from alvin_cli.utils.helper_functions import extract_dict
from alvin_cli.utils.helper_functions import format_response_data
from alvin_cli.utils.helper_functions import handle_print_exception
from alvin_cli.utils.helper_functions import print_output_format

app = typer.Typer(help="Connect Data Platforms with Alvin")

fields_to_print = [
    "id",
    "name",
    "platform_type",
    "created",
    "is_syncing",
    "last_updated",
    "additional_config",
]


@app.command()
def bigquery(
    name: str = typer.Option(
        ..., help=typer.style("Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    display_name: str = typer.Option(
        ..., help=typer.style("Display Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    service_account_json: str = typer.Option(
        ...,
        help=typer.style(
            "JSON enclosed in single quotes", fg=typer.colors.BRIGHT_GREEN, bold=True
        ),
    ),
    ignored_databases: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored databases as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_schemas: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored schemas as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_query_types: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored query types as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_users: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored users as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_query_statuses: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored query statuses as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    only_validate: bool = typer.Option(
        False,
        help=typer.style("Test the connection", fg=typer.colors.CYAN, bold=True),
    ),
) -> None:
    """Add bigquery platform"""

    request_in = DataPlatformBigQueryCreate(
        name=name,
        display_name=display_name,
        credentials=BigQueryAccountDetails(service_account_json),
        additional_config=BigQueryAdditionalConfig(
            ignored_databases=json.loads(ignored_databases),
            ignored_schemas=json.loads(ignored_schemas),
            ignored_query_types=json.loads(ignored_query_types),
            ignored_users=json.loads(ignored_users),
            ignored_query_statuses=json.loads(ignored_query_statuses),
        ),
    )

    try:
        create_response = (
            default_api.create_bigquery_platform_api_v1_platforms_bigquery_post(
                request_in
            )
        )

        if only_validate:
            typer.secho("TESTING THE CONNECTION", fg=typer.colors.GREEN)
            default_api.test_bigquery_platform_api_v1_platforms_bigquery_test_post(
                request_in
            )

            typer.secho("TEST SUCCESSFUL", fg=typer.colors.GREEN)

    except Exception as e:

        typer.secho("CONNECTION FAILED", fg=typer.colors.RED)

        exception = e.__str__()

        handle_print_exception(extract_dict(exception), exception[:5])

        return

    typer.secho("CREATING NEW CONNECTION", fg=typer.colors.CYAN)

    structured_data = format_response_data(fields_to_print, [create_response])

    print_output_format(structured_data, f"{name} platform", "table", False, "")

    typer.secho("CONNECTION SUCCESSFULLY CREATED", fg=typer.colors.MAGENTA)


@app.command()
def looker(
    name: str = typer.Option(
        ..., help=typer.style("Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    display_name: str = typer.Option(
        ..., help=typer.style("Display Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    client_id: str = typer.Option(
        ..., help=typer.style("Client Id", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    client_secret: str = typer.Option(
        ...,
        help=typer.style(
            "Client Secret/Password", fg=typer.colors.BRIGHT_GREEN, bold=True
        ),
    ),
    base_url: str = typer.Option(
        ..., help=typer.style("Base URL", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    ignored_projects: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored projects as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_models: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored models as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_folders: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored folders as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    data_source_platform_id: str = typer.Option(
        "", help=typer.style("Data Source Platform ID", fg=typer.colors.YELLOW)
    ),
    only_validate: bool = typer.Option(
        False,
        help=typer.style("Test the connection", fg=typer.colors.CYAN, bold=True),
    ),
) -> None:
    """Add looker platform"""

    request_in = DataPlatformLookerCreate(
        name=name,
        display_name=display_name,
        credentials=LookerCredentials(
            base_url=base_url, client_id=client_id, client_secret=client_secret
        ),
        additional_config=LookerAdditionalConfig(
            ignored_projects=json.loads(ignored_projects),
            ignored_models=json.loads(ignored_models),
            ignored_folders=json.loads(ignored_folders),
            data_source_platform_id=data_source_platform_id,
        ),
    )

    try:
        create_response = (
            default_api.create_looker_platform_api_v1_platforms_looker_post(request_in)
        )

        if only_validate:

            default_api.test_looker_platform_api_v1_platforms_looker_test_post(
                request_in
            )

            typer.secho("TEST SUCCESSFUL", fg=typer.colors.GREEN)

    except Exception as e:
        typer.secho("CONNECTION FAILED", fg=typer.colors.RED)
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return

    typer.secho("CREATING NEW CONNECTION", fg=typer.colors.CYAN)
    structured_data = format_response_data(fields_to_print, [create_response])
    print_output_format(structured_data, f"{name} platform", "table", False, "")
    typer.secho("CONNECTION SUCCESSFULLY CREATED", fg=typer.colors.MAGENTA)


@app.command()
def mode(
    name: str = typer.Option(
        ..., help=typer.style("Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    display_name: str = typer.Option(
        ..., help=typer.style("Display Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    access_token: str = typer.Option(
        ..., help=typer.style("Access Token", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    password: str = typer.Option(
        ..., help=typer.style("Password", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    workspace_username: str = typer.Option(
        "",
        help=typer.style("Workspace Username", fg=typer.colors.BRIGHT_GREEN, bold=True),
    ),
    ignored_spaces_tokens: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored space tokens as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignore_private_spaces: bool = typer.Option(
        False,
        help=typer.style("Ignore private spaces", fg=typer.colors.YELLOW),
    ),
    only_validate: bool = typer.Option(
        False,
        help=typer.style("Test the connection", fg=typer.colors.CYAN, bold=True),
    ),
) -> None:
    """Add Mode platform"""

    request_in = DataPlatformModeCreate(
        name=name,
        display_name=display_name,
        credentials=ModeCredentials(access_token=access_token, password=password),
        additional_config=ModeAdditionalConfig(
            workspace_username=workspace_username,
            ignored_spaces_tokens=ignored_spaces_tokens,
            ignore_private_spaces=ignore_private_spaces,
        ),
    )

    try:
        create_response = default_api.create_mode_platform_api_v1_platforms_mode_post(
            request_in
        )

        if only_validate:
            typer.secho("TESTING THE CONNECTION", fg=typer.colors.GREEN)

            default_api.test_mode_platform_api_v1_platforms_mode_test_post(request_in)

            typer.secho("TEST SUCCESSFUL", fg=typer.colors.GREEN)

    except Exception as e:

        typer.secho("CONNECTION FAILED", fg=typer.colors.RED)

        exception = e.__str__()

        handle_print_exception(extract_dict(exception), exception[:5])

        return

    typer.secho("CREATING NEW CONNECTION", fg=typer.colors.CYAN)

    structured_data = format_response_data(fields_to_print, [create_response])

    print_output_format(structured_data, f"{name} platform", "table", False, "")

    typer.secho("CONNECTION SUCCESSFULLY CREATED", fg=typer.colors.MAGENTA)


@app.command()
def redshift(
    name: str = typer.Option(
        ..., help=typer.style("Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    display_name: str = typer.Option(
        ..., help=typer.style("Display Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    host: str = typer.Option(
        ..., help=typer.style("Host", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    port: int = typer.Option(
        ..., help=typer.style("Port", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    username: str = typer.Option(
        ..., help=typer.style("username", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    password: str = typer.Option(
        ..., help=typer.style("Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    database: str = typer.Option(
        ..., help=typer.style("Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    ignored_databases: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored databases as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_schemas: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored schemas as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_query_types: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored query types as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_users: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored users as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_query_statuses: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored query statuses as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    only_validate: bool = typer.Option(
        False,
        help=typer.style("Test the connection", fg=typer.colors.CYAN, bold=True),
    ),
) -> None:
    """Add Redshift platform"""

    request_in = DataPlatformRedshiftCreate(
        name=name,
        display_name=display_name,
        credentials=RedshiftAccountDetails(
            host=host,
            port=port,
            username=username,
            password=password,
            database=database,
        ),
        additional_config=RedshiftAdditionalConfig(
            ignored_databases=json.loads(ignored_databases),
            ignored_schemas=json.loads(ignored_schemas),
            ignored_query_types=json.loads(ignored_query_types),
            ignored_users=json.loads(ignored_users),
            ignored_query_statuses=json.loads(ignored_query_statuses),
        ),
    )

    try:
        create_response = (
            default_api.create_redshift_platform_api_v1_platforms_redshift_post(
                request_in
            )
        )

        if only_validate:
            typer.secho("TESTING THE CONNECTION", fg=typer.colors.GREEN)
            default_api.test_redshift_platform_api_v1_platforms_redshift_test_post(
                request_in
            )

            typer.secho("TEST SUCCESSFUL", fg=typer.colors.GREEN)

    except Exception as e:

        typer.secho("CONNECTION FAILED", fg=typer.colors.RED)

        exception = e.__str__()

        handle_print_exception(extract_dict(exception), exception[:5])

        return

    typer.secho("CREATING NEW CONNECTION", fg=typer.colors.CYAN)

    structured_data = format_response_data(fields_to_print, [create_response])

    print_output_format(structured_data, f"{name} platform", "table", False, "")

    typer.secho("CONNECTION SUCCESSFULLY CREATED", fg=typer.colors.MAGENTA)


@app.command()
def snowflake(
    name: str = typer.Option(
        ..., help=typer.style("Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    display_name: str = typer.Option(
        ..., help=typer.style("Display Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    account: str = typer.Option(
        ..., help=typer.style("Account", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    username: str = typer.Option(
        ..., help=typer.style("Username", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    password: str = typer.Option(
        ..., help=typer.style("Password", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    role: str = typer.Option(
        ..., help=typer.style("Role", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    warehouse: str = typer.Option(
        ..., help=typer.style("Warehouse", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    ignored_databases: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored databases as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_schemas: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored schemas as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_query_types: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored query types as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_users: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored users as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
            bold=True,
        ),
    ),
    ignored_query_statuses: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored query statuses as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    ignored_roles: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored roles as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    only_validate: bool = typer.Option(
        False,
        help=typer.style("Test the connection", fg=typer.colors.CYAN, bold=True),
    ),
) -> None:
    """Add Snowflake platform"""

    request_in = DataPlatformSnowflakeCreate(
        name=name,
        display_name=display_name,
        credentials=SnowflakeAccountDetails(
            account=account,
            region=account.split(".")[-1],
            username=username,
            password=password,
            role=role,
            warehouse=warehouse,
        ),
        additional_config=SnowflakeAdditionalConfig(
            ignored_databases=json.loads(ignored_databases),
            ignored_schemas=json.loads(ignored_schemas),
            ignored_query_types=json.loads(ignored_query_types),
            ignored_users=json.loads(ignored_users),
            ignored_query_statuses=json.loads(ignored_query_statuses),
            ignored_roles=json.loads(ignored_roles),
        ),
    )
    try:
        create_response = (
            default_api.create_snowflake_platform_api_v1_platforms_snowflake_post(
                request_in
            )
        )

        if only_validate:
            typer.secho("TESTING THE CONNECTION", fg=typer.colors.GREEN)
            default_api.test_snowflake_platform_api_v1_platforms_snowflake_test_post(
                request_in
            )
            typer.secho("TEST SUCCESSFUL", fg=typer.colors.GREEN)

    except Exception as e:

        typer.secho("CONNECTION FAILED", fg=typer.colors.RED)

        exception = e.__str__()

        handle_print_exception(extract_dict(exception), exception[:5])

        return

    typer.secho("CREATING NEW CONNECTION", fg=typer.colors.CYAN)

    structured_data = format_response_data(fields_to_print, [create_response])

    print_output_format(structured_data, f"{name} platform", "table", False, "")

    typer.secho("CONNECTION SUCCESSFULLY CREATED", fg=typer.colors.MAGENTA)


@app.command()
def tableau(
    name: str = typer.Option(
        ..., help=typer.style("Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    display_name: str = typer.Option(
        ..., help=typer.style("Display Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    base_url: str = typer.Option(
        ..., help=typer.style("Base URL", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    api_base_url: str = typer.Option(
        ..., help=typer.style("API Base URL", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    api_version: str = typer.Option(
        ..., help=typer.style("API Version", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    site_name: str = typer.Option(
        ..., help=typer.style("Site Name", fg=typer.colors.BRIGHT_GREEN, bold=True)
    ),
    access_token_name: str = typer.Option(
        ...,
        help=typer.style("Access Token Name", fg=typer.colors.BRIGHT_GREEN, bold=True),
    ),
    access_token_secret: str = typer.Option(
        ...,
        help=typer.style(
            "Access Token Secret", fg=typer.colors.BRIGHT_GREEN, bold=True
        ),
    ),
    ignored_projects: str = typer.Option(
        "[]",
        help=typer.style(
            "Ignored projects as single quotes enclosed list of strings",
            fg=typer.colors.YELLOW,
        ),
    ),
    only_validate: bool = typer.Option(
        False,
        help=typer.style("Test the connection", fg=typer.colors.CYAN, bold=True),
    ),
) -> None:
    """Add Tableau platform"""
    request_in = DataPlatformTableauCreate(
        name=name,
        display_name=display_name,
        credentials=TableauCredentials(
            base_url=base_url,
            api_base_url=api_base_url,
            api_version=api_version,
            site_name=site_name,
            access_token_name=access_token_name,
            access_token_secret=access_token_secret,
        ),
        additional_config=TableauAdditionalConfig(
            ignored_projects=json.loads(ignored_projects)
        ),
    )
    try:
        create_response = (
            default_api.create_tableau_platform_api_v1_platforms_tableau_post(
                request_in
            )
        )

        if only_validate:
            typer.secho("TESTING THE CONNECTION", fg=typer.colors.GREEN)
            default_api.test_tableau_platform_api_v1_platforms_tableau_test_post(
                request_in
            )

            typer.secho("TEST SUCCESSFUL", fg=typer.colors.GREEN)

    except Exception as e:

        typer.secho("CONNECTION FAILED", fg=typer.colors.RED)

        exception = e.__str__()

        handle_print_exception(extract_dict(exception), exception[:5])

        return

    typer.secho("CREATING NEW CONNECTION", fg=typer.colors.CYAN)

    structured_data = format_response_data(fields_to_print, [create_response])

    print_output_format(structured_data, f"{name} platform", "table", False, "")

    typer.secho("CONNECTION SUCCESSFULLY CREATED", fg=typer.colors.MAGENTA)
