import typer

from alvin_cli.platforms import add as add_methods
from alvin_cli.schemas.models import OutputFormat
from alvin_cli.utils import default_api
from alvin_cli.utils.helper_functions import extract_dict
from alvin_cli.utils.helper_functions import format_response_data
from alvin_cli.utils.helper_functions import handle_print_exception
from alvin_cli.utils.helper_functions import print_output_format

app = typer.Typer()

app.add_typer(add_methods.app, name="add")

fields_to_print = [
    "id",
    "name",
    "platform_type",
    "created",
    "is_syncing",
    "last_updated",
    "additional_config",
]


@app.command()
def list(
    output: OutputFormat = typer.Option(
        OutputFormat.table,
        help=typer.style(
            "Format to receive output in, default is table", fg=typer.colors.YELLOW
        ),
    ),
    save_to_file: bool = typer.Option(
        False, help=typer.style("Save data to a file?", fg=typer.colors.YELLOW)
    ),
    file_name: str = typer.Option(
        "platforms_list",
        help=typer.style(
            "File Name if file save selected. 'platforms_list.* by default",
            fg=typer.colors.YELLOW,
        ),
    ),
) -> None:
    """List all platforms"""

    try:
        table_title = "List Of Platforms"
        platforms_list = default_api.get_platforms_api_v1_platforms_get()
        structured_data = format_response_data(fields_to_print, platforms_list)
        print_output_format(
            structured_data, table_title, output, save_to_file, file_name
        )

    except Exception as e:
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return


@app.command()
def get(
    platform_id: str = typer.Option(
        ...,
        help=typer.style(
            "Platform ID To Get Details", fg=typer.colors.BRIGHT_GREEN, bold=True
        ),
    ),
    output: OutputFormat = typer.Option(
        OutputFormat.table,
        help=typer.style(
            "Format to receive output in, default is table", fg=typer.colors.YELLOW
        ),
    ),
    save_to_file: bool = typer.Option(
        False, help=typer.style("Save data to a file?", fg=typer.colors.YELLOW)
    ),
    file_name: str = typer.Option(
        f"platform",
        help=typer.style(
            "File Name if file save selected. 'platform.* by default",
            fg=typer.colors.YELLOW,
        ),
    ),
) -> None:
    """Get platform details"""

    try:
        platform_response = default_api.get_platform_api_v1_platforms_platform_id_get(
            platform_id
        )

        if platform_response:
            table_title = f"{platform_id} Details"
            structured_data = format_response_data(fields_to_print, [platform_response])
            print_output_format(
                structured_data, table_title, output, save_to_file, file_name
            )
        else:
            typer.secho("Check arguments are valid", fg=typer.colors.BRIGHT_RED)

    except Exception as e:
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return


@app.command()
def delete(
    platform_id: str = typer.Option(
        ...,
        help=typer.style(
            "Platform ID To Delete", fg=typer.colors.BRIGHT_GREEN, bold=True
        ),
    )
) -> None:
    """Delete platform"""

    try:
        default_api.delete_platform_api_v1_platforms_platform_id_delete(platform_id)
        typer.secho(
            f"We have noted your request to delete the platform {platform_id}!",
            fg=typer.colors.MAGENTA,
        )

    except Exception as e:
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return


@app.command()
def sync(
    platform_id: str = typer.Option(
        ..., help=typer.style("Platform ID To Sync", fg=typer.colors.BRIGHT_GREEN)
    )
) -> None:
    """Sync platform"""

    try:
        default_api.sync_platform_rpc_api_v1_platforms_platform_id_rpc_sync_get(
            platform_id
        )

        typer.secho(f"{platform_id} sync scheduled!", fg=typer.colors.CYAN)

    except Exception as e:
        exception = e.__str__()
        handle_print_exception(extract_dict(exception), exception[:5])
        return
